# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import copy
from unittest import mock

from vmware_nsxlib.tests.unit.v3 import nsxlib_testcase
from vmware_nsxlib.tests.unit.v3.policy import test_resources
from vmware_nsxlib.v3 import exceptions as nsxlib_exc
from vmware_nsxlib.v3 import nsx_constants
from vmware_nsxlib.v3.policy import constants
from vmware_nsxlib.v3.policy import lb_defs
TEST_TENANT = 'test'


class TestPolicyLBClientSSLProfileApi(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBClientSSLProfileApi, self).setUp()
        self.resourceApi = self.policy_lib.load_balancer.client_ssl_profile

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        protocols = ['TLS_V1_1']
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                client_ssl_profile_id=obj_id,
                description=description,
                protocols=protocols,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBClientSslProfileDef(
                client_ssl_profile_id=obj_id,
                name=name,
                description=description,
                protocols=protocols,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBClientSslProfileDef(
                client_ssl_profile_id=mock.ANY,
                name=name,
                description=description,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBClientSslProfileDef(
                client_ssl_profile_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBClientSslProfileDef(
                client_ssl_profile_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = lb_defs.LBClientSslProfileDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = lb_defs.LBClientSslProfileDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        with self.mock_get(obj_id, name), \
            self.mock_create_update() as update_call:
            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    tenant=TEST_TENANT)
            expected_def = lb_defs.LBClientSslProfileDef(
                client_ssl_profile_id=obj_id,
                name=name,
                description=description,
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)


class TestPolicyLBServerSSLProfileApi(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBServerSSLProfileApi, self).setUp()
        self.resourceApi = self.policy_lib.load_balancer.server_ssl_profile

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        protocols = ['TLS_V1_1']
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                server_ssl_profile_id=obj_id,
                description=description,
                protocols=protocols,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBServerSslProfileDef(
                server_ssl_profile_id=obj_id,
                name=name,
                description=description,
                protocols=protocols,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBServerSslProfileDef(
                server_ssl_profile_id=mock.ANY,
                name=name,
                description=description,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBServerSslProfileDef(
                server_ssl_profile_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBServerSslProfileDef(
                server_ssl_profile_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = lb_defs.LBServerSslProfileDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = lb_defs.LBServerSslProfileDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        with self.mock_get(obj_id, name), \
            self.mock_create_update() as update_call:
            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    tenant=TEST_TENANT)
            expected_def = lb_defs.LBServerSslProfileDef(
                server_ssl_profile_id=obj_id,
                name=name,
                description=description,
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)


class TestPolicyLBPersistenceProfile(
    test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBPersistenceProfile, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_persistence_profile)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = (
                self.resourceApi.entry_def(
                    persistence_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = (
                self.resourceApi.entry_def(
                    persistence_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = (
                self.resourceApi.entry_def(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = (
                self.resourceApi.entry_def(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_wait_until_realized_fail(self):
        pers_id = 'test_pers'
        info = {'state': constants.STATE_UNREALIZED,
                'realization_specific_identifier': pers_id}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            self.assertRaises(nsxlib_exc.RealizationTimeoutError,
                              self.resourceApi.wait_until_realized,
                              pers_id, max_attempts=5, sleep=0.1,
                              tenant=TEST_TENANT)

    def test_wait_until_realized_succeed(self):
        pers_id = 'test_pers'
        info = {'state': constants.STATE_REALIZED,
                'realization_specific_identifier': pers_id,
                'entity_type': 'LbPersistenceProfileDto'}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            actual_info = self.resourceApi.wait_until_realized(
                pers_id, entity_type='LbPersistenceProfileDto', max_attempts=5,
                sleep=0.1, tenant=TEST_TENANT)
            self.assertEqual(info, actual_info)


class TestPolicyLBCookiePersistenceProfile(
    test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBCookiePersistenceProfile, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_cookie_persistence_profile)

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        cookie_garble = 'test_garble'
        cookie_name = 'test_name'
        cookie_mode = 'INSERT'
        cookie_path = 'path'
        cookie_time = 'time'
        persistence_shared = False
        with self.mock_create_update() as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                persistence_profile_id=obj_id,
                description=description,
                cookie_name=cookie_name,
                cookie_garble=cookie_garble,
                cookie_mode=cookie_mode,
                cookie_path=cookie_path,
                cookie_time=cookie_time,
                persistence_shared=persistence_shared,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBCookiePersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    name=name,
                    description=description,
                    cookie_name=cookie_name,
                    cookie_garble=cookie_garble,
                    cookie_mode=cookie_mode,
                    cookie_path=cookie_path,
                    cookie_time=cookie_time,
                    persistence_shared=persistence_shared,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with self.mock_create_update() as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBCookiePersistenceProfileDef(
                    persistence_profile_id=mock.ANY,
                    name=name,
                    description=description,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBCookiePersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBCookiePersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [
                {'resource_type': self.resourceApi.entry_def.resource_type(),
                 'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = (
                lb_defs.LBCookiePersistenceProfileDef(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [
                {'resource_type': self.resourceApi.entry_def.resource_type(),
                 'display_name': 'profile1'},
                {'resource_type': 'wrong_type',
                 'display_name': 'profile2'}]}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBCookiePersistenceProfileDef(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(1, len(result))

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        cookie_garble = 'test_garble'
        cookie_name = 'test_name'
        cookie_mode = 'INSERT'
        cookie_path = 'path'
        cookie_time = 'time'
        persistence_shared = False
        with self.mock_get(obj_id, name), \
            self.mock_create_update() as update_call:
            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    cookie_name=cookie_name,
                                    cookie_garble=cookie_garble,
                                    cookie_mode=cookie_mode,
                                    cookie_path=cookie_path,
                                    cookie_time=cookie_time,
                                    persistence_shared=persistence_shared,
                                    tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBCookiePersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    name=name,
                    description=description,
                    cookie_name=cookie_name,
                    cookie_garble=cookie_garble,
                    cookie_mode=cookie_mode,
                    cookie_path=cookie_path,
                    cookie_time=cookie_time,
                    persistence_shared=persistence_shared,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(update_call, expected_def)


class TestPolicyLBSourceIpProfileApi(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBSourceIpProfileApi, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_source_ip_persistence_profile)

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        ha = 'ha'
        persistence_shared = True
        purge = 'purge'
        timeout = 100
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                persistence_profile_id=obj_id,
                description=description,
                ha_persistence_mirroring_enabled=ha,
                persistence_shared=persistence_shared,
                purge=purge,
                timeout=timeout,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    name=name,
                    description=description,
                    ha_persistence_mirroring_enabled=ha,
                    persistence_shared=persistence_shared,
                    purge=purge,
                    timeout=timeout,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    persistence_profile_id=mock.ANY,
                    name=name,
                    description=description,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [
                {'resource_type': self.resourceApi.entry_def.resource_type(),
                 'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        profiles = [{'resource_type': 'LBSourceIpPersistenceProfile',
                     'id': 'default-source-ip-lb-persistence-profile',
                     'display_name': 'default-source-ip-profile'}]
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': profiles}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(profiles, result)

    def test_list_different_type(self):
        profiles = [{'resource_type': 'LBSourceCookiePersistenceProfile',
                     'id': 'default-source-ip-lb-persistence-profile',
                     'display_name': 'default-source-ip-profile'}]
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': profiles}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_list_empty(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        ha = False
        persistence_shared = False
        purge = 'no purge'
        timeout = 101
        with self.mock_get(obj_id, name), \
            self.mock_create_update() as update_call:

            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    ha_persistence_mirroring_enabled=ha,
                                    persistence_shared=persistence_shared,
                                    purge=purge,
                                    timeout=timeout,
                                    tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBSourceIpPersistenceProfileDef(
                    persistence_profile_id=obj_id,
                    name=name,
                    description=description,
                    ha_persistence_mirroring_enabled=ha,
                    persistence_shared=persistence_shared,
                    purge=purge,
                    timeout=timeout,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(update_call, expected_def)


class TestPolicyLBApplicationProfile(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBApplicationProfile, self).setUp()
        self.resourceApi = self.policy_lib.load_balancer.lb_http_profile

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        http_redirect_to_https = False
        http_redirect_to = "sample-url"
        idle_timeout = 100
        ntlm = False
        request_body_size = 1025
        request_header_size = 10
        response_header_size = 10
        response_timeout = 10
        x_forwarded_for = 'INSERT'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                lb_app_profile_id=obj_id,
                description=description,
                http_redirect_to_https=http_redirect_to_https,
                http_redirect_to=http_redirect_to,
                idle_timeout=idle_timeout,
                ntlm=ntlm,
                request_body_size=request_body_size,
                request_header_size=request_header_size,
                response_header_size=response_header_size,
                response_timeout=response_timeout,
                x_forwarded_for=x_forwarded_for,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBHttpProfileDef(
                    lb_app_profile_id=obj_id,
                    name=name,
                    description=description,
                    http_redirect_to_https=http_redirect_to_https,
                    http_redirect_to=http_redirect_to,
                    idle_timeout=idle_timeout,
                    ntlm=ntlm,
                    request_body_size=request_body_size,
                    request_header_size=request_header_size,
                    response_header_size=response_header_size,
                    response_timeout=response_timeout,
                    x_forwarded_for=x_forwarded_for,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_fast_tcp_profile_def(self):
        obj_dict = {'close_timeout': 8,
                    'ha_flow_mirroring_enabled': False,
                    'idle_timeout': 100}
        fast_tcp_profile_def = lb_defs.LBFastTcpProfile(**obj_dict)
        self.assertDictContainsSubset(obj_dict,
                                      fast_tcp_profile_def.get_obj_dict())

    def test_fast_udp_profile_def(self):
        obj_dict = {'flow_mirroring_enabled': False,
                    'idle_timeout': 100}
        fast_udp_profile_def = lb_defs.LBFastUdpProfile(**obj_dict)
        self.assertDictContainsSubset(obj_dict,
                                      fast_udp_profile_def.get_obj_dict())

    def test_http_profile_def(self):
        obj_dict = {'http_redirect_to_https': False,
                    'http_redirect_to': "sample-url",
                    'idle_timeout': 100,
                    'ntlm': False,
                    'request_body_size': 1025,
                    'request_header_size': 10,
                    'response_header_size': 10,
                    'response_timeout': 10,
                    'x_forwarded_for': 'INSERT'}
        http_profile_def = lb_defs.LBHttpProfileDef(**obj_dict)
        self.assertDictContainsSubset(obj_dict,
                                      http_profile_def.get_obj_dict())

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBHttpProfileDef(
                    lb_app_profile_id=mock.ANY,
                    name=name,
                    description=description,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBHttpProfileDef(
                    lb_app_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBHttpProfileDef(
                    lb_app_profile_id=obj_id,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = (
                lb_defs.LBHttpProfileDef(tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBHttpProfileDef(tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        with self.mock_get(obj_id, name), \
            self.mock_create_update() as update_call:
            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBHttpProfileDef(
                    lb_app_profile_id=obj_id,
                    name=name,
                    description=description,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(update_call, expected_def)


class TestPolicyLBService(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBService, self).setUp()
        self.resourceApi = self.policy_lib.load_balancer.lb_service

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        size = 'SMALL'
        connectivity_path = 'path'
        relax_scale_validation = True
        with self.mock_create_update() as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                lb_service_id=obj_id,
                description=description,
                size=size,
                connectivity_path=connectivity_path,
                relax_scale_validation=relax_scale_validation,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBServiceDef(
                    nsx_version=self.policy_lib.get_version(),
                    lb_service_id=obj_id,
                    name=name,
                    description=description,
                    size=size,
                    connectivity_path=connectivity_path,
                    relax_scale_validation=relax_scale_validation,
                    tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with self.mock_create_update() as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBServiceDef(lb_service_id=mock.ANY,
                                     name=name,
                                     description=description,
                                     tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_create_with_unsupported_attribute(self):
        name = 'd1'
        description = 'desc'
        relax_scale_validation = True

        with self.mock_create_update() as api_call, \
                mock.patch.object(self.resourceApi, 'version', '0.0.0'):
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                relax_scale_validation=relax_scale_validation,
                tenant=TEST_TENANT)
            expected_def = (
                lb_defs.LBServiceDef(lb_service_id=mock.ANY,
                                     name=name,
                                     description=description,
                                     tenant=TEST_TENANT))
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBServiceDef(
                lb_service_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBServiceDef(
                lb_service_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = lb_defs.LBServiceDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def _test_list(self, silent=False, silent_if_empty=False):
        s1 = {'id': 'xxx', 'display_name': 'yyy'}
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': [s1]}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT, silent=silent,
                                           silent_if_empty=silent_if_empty)
            expected_def = lb_defs.LBServiceDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([s1], result)

    def test_list(self):
        self._test_list()

    def test_list_total_silence(self):
        self._test_list(silent=True)

    def test_list_silent_if_empty(self):
        self._test_list(silent_if_empty=True)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        size = 'SMALL'
        connectivity_path = 'path'
        relax_scale_validation = True
        with self.mock_get(obj_id, name), \
            self.mock_create_update() as update_call:
            self.resourceApi.update(
                obj_id,
                name=name,
                description=description,
                tenant=TEST_TENANT,
                size=size,
                connectivity_path=connectivity_path,
                relax_scale_validation=relax_scale_validation)
            expected_def = lb_defs.LBServiceDef(
                nsx_version=self.policy_lib.get_version(),
                lb_service_id=obj_id,
                name=name,
                description=description,
                tenant=TEST_TENANT,
                size=size,
                connectivity_path=connectivity_path,
                relax_scale_validation=relax_scale_validation)
            self.assert_called_with_def(update_call, expected_def)

    def test_update_customized(self):
        obj_id = '111'
        name = 'name'
        tags = [{'tag': '1', 'scope': '2'}]

        def update_callback(body):
            body['tags'] = tags

        with self.mock_get(obj_id, name), \
            mock.patch.object(self.policy_api.client, "update") as update_call:
            self.resourceApi.update_customized(
                obj_id, update_callback)

            update_call.assert_called_once_with(
                'infra/lb-services/%s' % obj_id,
                {'id': obj_id, 'display_name': name,
                 'resource_type': 'LBService',
                 'tags': tags})

    def test_get_status(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get") as api_call:
            self.resourceApi.get_status(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBServiceStatusDef(
                lb_service_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get_statistics(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get") as api_call:
            self.resourceApi.get_statistics(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBServiceStatisticsDef(
                lb_service_id=obj_id,
                realtime=False,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual('%s/lb-services/%s/statistics/',
                             expected_def.path_pattern)

    def test_get_statistics_realtime(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get") as api_call:
            self.resourceApi.get_statistics(obj_id, realtime=True,
                                            tenant=TEST_TENANT)
            expected_def = lb_defs.LBServiceStatisticsDef(
                lb_service_id=obj_id,
                realtime=True,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual('%s/lb-services/%s/statistics?source=realtime',
                             expected_def.path_pattern)

    def test_get_virtual_server_status(self):
        obj_id = '111'
        vs_id = '222'
        with mock.patch.object(self.policy_api, "get") as api_call:
            self.resourceApi.get_virtual_server_status(
                obj_id, vs_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBVirtualServerStatusDef(
                lb_service_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get_usage(self):
        lbs_id = 'test_vs'
        with mock.patch.object(self.policy_api, "get") as api_call:
            self.resourceApi.get_usage(
                lbs_id, realtime=True, tenant=TEST_TENANT)
            expected_def = lb_defs.LBServiceUsageDef(
                lb_service_id=lbs_id,
                realtime=True,
                tenant=TEST_TENANT)
            expected_path = '%s/lb-services/%s/service-usage?source=realtime'
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(expected_def.path_pattern, expected_path)

    def test_wait_until_realized_fail(self):
        lbs_id = 'test_lbs'
        info = {'state': constants.STATE_UNREALIZED,
                'realization_specific_identifier': lbs_id,
                'entity_type': 'LbServiceDto'}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            self.assertRaises(nsxlib_exc.RealizationTimeoutError,
                              self.resourceApi.wait_until_realized,
                              lbs_id, max_attempts=5, sleep=0.1,
                              tenant=TEST_TENANT)

    def test_wait_until_realized_error(self):
        lbs_id = 'test_lbs'
        error_code = 23500
        related_error_code = 23707
        error_msg = 'Found errors in the request.'
        related_error_msg = 'Exceed maximum number of load balancer.'
        info = {'state': constants.STATE_ERROR,
                'realization_specific_identifier': lbs_id,
                'entity_type': 'LbServiceDto',
                'alarms': [{
                    'message': error_msg,
                    'error_details': {
                        'related_errors': [{
                            'error_code': related_error_code,
                            'module_name': 'LOAD-BALANCER',
                            'error_message': related_error_msg
                        }],
                        'error_code': error_code,
                        'module_name': 'LOAD-BALANCER',
                        'error_message': error_msg
                    }
                }]}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            with self.assertRaises(nsxlib_exc.RealizationErrorStateError) as e:
                self.resourceApi.wait_until_realized(
                    lbs_id, tenant=TEST_TENANT)
            error_msg_tail = "%s: %s" % (error_msg, related_error_msg)
            self.assertTrue(e.exception.msg.endswith(error_msg_tail))
            self.assertEqual(e.exception.error_code, error_code)
            self.assertEqual(e.exception.related_error_codes,
                             [related_error_code])

    def test_wait_until_realized_succeed(self):
        lbs_id = 'test_lbs'
        info = {'state': constants.STATE_REALIZED,
                'realization_specific_identifier': lbs_id,
                'entity_type': 'LbServiceDto'}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            actual_info = self.resourceApi.wait_until_realized(
                lbs_id, max_attempts=5, sleep=0.1, tenant=TEST_TENANT)
            self.assertEqual(info, actual_info)


class TestPolicyLBVirtualServer(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBVirtualServer, self).setUp()
        self.resourceApi = self.policy_lib.load_balancer.virtual_server

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        waf_profile_id = 'waf'
        waf_profile_path = self.policy_lib.waf_profile.get_path(
            profile_id=waf_profile_id, tenant=TEST_TENANT)
        waf_profile_binding = lb_defs.WAFProfileBindingDef(
            waf_profile_path=waf_profile_path)
        lb_acl = self.resourceApi.build_access_list_control(
            constants.ACTION_ALLOW, 'fake_group_path', True)
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                virtual_server_id=obj_id,
                waf_profile_binding=waf_profile_binding,
                description=description,
                access_list_control=lb_acl,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBVirtualServerDef(
                nsx_version=self.policy_lib.get_version(),
                virtual_server_id=obj_id, name=name, description=description,
                waf_profile_binding=waf_profile_binding,
                access_list_control=lb_acl.get_obj_dict(),
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=mock.ANY, name=name, description=description,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = lb_defs.LBVirtualServerDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = lb_defs.LBVirtualServerDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        vs_name = 'name-name'
        with self.mock_get(obj_id, vs_name), \
            self.mock_create_update() as update_call:

            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    tenant=TEST_TENANT)
            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=obj_id, name=name,
                description=description,
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)

    def test_update_log_parameters(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        vs_name = 'name-name'
        with self.mock_get(obj_id, vs_name), \
            self.mock_create_update() as update_call:
            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    tenant=TEST_TENANT,
                                    access_log_enabled=True,
                                    log_significant_event_only=True)
            expected_def = lb_defs.LBVirtualServerDef(
                nsx_version=nsx_constants.NSX_VERSION_3_0_0,
                virtual_server_id=obj_id, name=name,
                description=description,
                tenant=TEST_TENANT, access_log_enabled=True,
                log_significant_event_only=True)
            self.assert_called_with_def(update_call, expected_def)

    def test_log_parameters_for_version(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'

        expected_def = lb_defs.LBVirtualServerDef(
            nsx_version=nsx_constants.NSX_VERSION_2_5_0,
            virtual_server_id=obj_id, name=name,
            description=description,
            tenant=TEST_TENANT, access_log_enabled=True,
            log_significant_event_only=True)
        self.assertFalse('access_log_enabled' in expected_def.get_obj_dict())
        self.assertFalse('log_significant_event_only' in
                         expected_def.get_obj_dict())

        expected_def = lb_defs.LBVirtualServerDef(
            nsx_version=nsx_constants.NSX_VERSION_3_0_0,
            virtual_server_id=obj_id, name=name,
            description=description,
            tenant=TEST_TENANT, access_log_enabled=True,
            log_significant_event_only=True)
        self.assertTrue('access_log_enabled' in expected_def.get_obj_dict())
        self.assertTrue('log_significant_event_only' in
                        expected_def.get_obj_dict())

    def test_non_partial_update(self):
        obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(obj_id, vs_name, max_concurrent_connections=80), \
            self.mock_create_update() as update_call:
            self.resourceApi.update(obj_id,
                                    max_concurrent_connections=None,
                                    tenant=TEST_TENANT,
                                    allow_partial_updates=False)
            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=obj_id, name=vs_name,
                max_concurrent_connections=None,
                tenant=TEST_TENANT)
            update_call.assert_called_with(mock.ANY, partial_updates=False,
                                           force=False)
            self.assert_called_with_def(update_call, expected_def)

    def test_add_lb_rule(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        rule_actions = 'test1'
        rule_match_conditions = 'test2'
        rule_name = 'dummy_rule'
        rule_match_strategy = 'test3'
        rule_phase = 'test4'
        with self.mock_get(vs_obj_id, vs_name), \
            self.mock_create_update() as update_call:
            self.resourceApi.add_lb_rule(
                vs_obj_id, actions=rule_actions, name=rule_name,
                match_conditions=rule_match_conditions,
                match_strategy=rule_match_strategy, phase=rule_phase)
            lb_rule = lb_defs.LBRuleDef(
                rule_actions, rule_match_conditions, rule_name,
                rule_match_strategy, rule_phase)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[lb_rule])
            self.assert_called_with_def(update_call, expected_def)

    def test_add_lb_rule_first(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        rule_actions = 'test1'
        rule_match_conditions = 'test2'
        rule_name = 'dummy_rule'
        rule_match_strategy = 'test3'
        rule_phase = 'test4'
        with self.mock_get(vs_obj_id, vs_name,
                           rules=[{'display_name': 'xx'},
                                  {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:

            self.resourceApi.add_lb_rule(
                vs_obj_id, actions=rule_actions, name=rule_name,
                match_conditions=rule_match_conditions,
                match_strategy=rule_match_strategy, phase=rule_phase,
                position=0)
            lb_rule = lb_defs.LBRuleDef(
                rule_actions, rule_match_conditions, rule_name,
                rule_match_strategy, rule_phase)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[lb_rule,
                       {'display_name': 'xx'},
                       {'display_name': 'yy'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_add_lb_rule_last(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        rule_actions = 'test1'
        rule_match_conditions = 'test2'
        rule_name = 'dummy_rule'
        rule_match_strategy = 'test3'
        rule_phase = 'test4'
        with self.mock_get(vs_obj_id, vs_name,
                           rules=[{'display_name': 'xx'},
                                  {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:
            self.resourceApi.add_lb_rule(
                vs_obj_id, actions=rule_actions, name=rule_name,
                match_conditions=rule_match_conditions,
                match_strategy=rule_match_strategy, phase=rule_phase)
            lb_rule = lb_defs.LBRuleDef(
                rule_actions, rule_match_conditions, rule_name,
                rule_match_strategy, rule_phase)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'xx'},
                       {'display_name': 'yy'},
                       lb_rule])
            self.assert_called_with_def(update_call, expected_def)

    def test_add_lb_rule_last_over(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        rule_actions = 'test1'
        rule_match_conditions = 'test2'
        rule_name = 'dummy_rule'
        rule_match_strategy = 'test3'
        rule_phase = 'test4'
        with self.mock_get(vs_obj_id, vs_name,
                           rules=[{'display_name': 'xx'},
                                  {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:

            self.resourceApi.add_lb_rule(
                vs_obj_id, actions=rule_actions, name=rule_name,
                match_conditions=rule_match_conditions,
                match_strategy=rule_match_strategy, phase=rule_phase,
                position=999)
            lb_rule = lb_defs.LBRuleDef(
                rule_actions, rule_match_conditions, rule_name,
                rule_match_strategy, rule_phase)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'xx'},
                       {'display_name': 'yy'},
                       lb_rule])
            self.assert_called_with_def(update_call, expected_def)

    def test_add_lb_rule_mid(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        rule_actions = 'test1'
        rule_match_conditions = 'test2'
        rule_name = 'dummy_rule'
        rule_match_strategy = 'test3'
        rule_phase = 'test4'
        with self.mock_get(vs_obj_id, vs_name,
                           rules=[{'display_name': 'xx'},
                                  {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:
            self.resourceApi.add_lb_rule(
                vs_obj_id, actions=rule_actions, name=rule_name,
                match_conditions=rule_match_conditions,
                match_strategy=rule_match_strategy, phase=rule_phase,
                position=1)
            lb_rule = lb_defs.LBRuleDef(
                rule_actions, rule_match_conditions, rule_name,
                rule_match_strategy, rule_phase)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'xx'},
                       lb_rule,
                       {'display_name': 'yy'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_update_lb_rule(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(
                vs_obj_id, vs_name,
                rules=[{'display_name': 'xx', 'actions': '11'},
                       {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:
            self.resourceApi.update_lb_rule(vs_obj_id, 'xx', actions='22')

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'xx', 'actions': '22'},
                       {'display_name': 'yy'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_update_lb_rule_position(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(
                vs_obj_id, vs_name,
                rules=[{'display_name': 'xx', 'actions': '11'},
                       {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:
            self.resourceApi.update_lb_rule(vs_obj_id, 'xx', actions='22',
                                            position=1)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'yy'},
                       {'display_name': 'xx', 'actions': '22'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_update_lb_rule_suffix(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(
                vs_obj_id, vs_name,
                rules=[{'display_name': 'xx_with_suffix', 'actions': '11'},
                       {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:
            self.resourceApi.update_lb_rule(
                vs_obj_id, 'xx', actions='22',
                compare_name_suffix='suffix')

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'xx', 'actions': '22'},
                       {'display_name': 'yy'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_update_lb_rule_wrong_suffix(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(
                vs_obj_id, vs_name,
                rules=[{'display_name': 'xx_with_suffix', 'actions': '11'},
                       {'display_name': 'yy'}]):
            self.assertRaises(nsxlib_exc.ResourceNotFound,
                              self.resourceApi.update_lb_rule,
                              vs_obj_id, 'xx', actions='22',
                              compare_name_suffix='bad')

    def test_remove_lb_rule(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(vs_obj_id, vs_name,
                           rules=[{'display_name': 'xx'},
                                  {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:
            self.resourceApi.remove_lb_rule(vs_obj_id, 'xx')

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'yy'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_remove_lb_rule_by_suffix(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(vs_obj_id, vs_name,
                           rules=[{'display_name': 'xx_with_suffix'},
                                  {'display_name': 'yy'}]), \
            self.mock_create_update() as update_call:
            self.resourceApi.remove_lb_rule(vs_obj_id, 'with_suffix',
                                            check_name_suffix=True)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'yy'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_remove_lb_rule_wrong_suffix(self):
        vs_obj_id = '111'
        vs_name = 'name-name'
        with self.mock_get(vs_obj_id, vs_name,
                           rules=[{'display_name': 'xx_with_suffix'},
                                  {'display_name': 'yy'}]),\
            self.mock_create_update() as update_call:
            self.resourceApi.remove_lb_rule(vs_obj_id, 'wrong_suffiX',
                                            check_name_suffix=True)

            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_obj_id,
                rules=[{'display_name': 'xx_with_suffix'},
                       {'display_name': 'yy'}])
            self.assert_called_with_def(update_call, expected_def)

    def test_build_access_list_control(self):
        lb_acl = self.resourceApi.build_access_list_control(
            constants.ACTION_ALLOW, 'fake_group_path', True)
        expected_acl_dict = {
            'action': constants.ACTION_ALLOW,
            'enabled': True,
            'group_path': 'fake_group_path'
        }
        self.assertDictEqual(lb_acl.get_obj_dict(), expected_acl_dict)

    def test_wait_until_realized_fail(self):
        vs_id = 'test_vs'
        info = {'state': constants.STATE_UNREALIZED,
                'realization_specific_identifier': vs_id}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            self.assertRaises(nsxlib_exc.RealizationTimeoutError,
                              self.resourceApi.wait_until_realized,
                              vs_id, max_attempts=5, sleep=0.1,
                              tenant=TEST_TENANT)

    def test_wait_until_realized_succeed(self):
        vs_id = 'test_vs'
        info = {'state': constants.STATE_REALIZED,
                'realization_specific_identifier': vs_id,
                'entity_type': 'LbVirtualServerDto'}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            actual_info = self.resourceApi.wait_until_realized(
                vs_id, entity_type='LbVirtualServerDto', max_attempts=5,
                sleep=0.1, tenant=TEST_TENANT)
            self.assertEqual(info, actual_info)

    def test_remove_virtual_server_client_ssl_profile_binding(self):
        vs_id = 'test-id'
        vs_name = 'test-name'
        client_binding = {
            'default_certificate_path': '/infra/certificates/test-cert',
            'client_ssl_profile_path': '/infra/lb-client-ssl-profiles/default'}
        server_binding = {
            'ssl_profile_path': '/infra/lb-server-ssl-profiles/test'}
        with self.mock_get(
                vs_id, vs_name, client_ssl_profile_binding=client_binding,
                server_ssl_profile_binding=server_binding), \
            self.mock_create_update() as update_call:
            self.resourceApi.remove_virtual_server_client_ssl_profile_binding(
                vs_id)
            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_id, name=vs_name)
            self.assert_called_with_def(update_call, expected_def)

    def test_remove_dlb_virtual_server_persistence_profile(self):
        vs_id = 'test-id'
        vs_name = 'test-name'
        with self.mock_get(
                vs_id, vs_name, lb_persistence_profile_path='test-profile'), \
                self.mock_create_update() as update_call:
            self.resourceApi.remove_dlb_virtual_server_persistence_profile(
                vs_id)
            expected_def = lb_defs.LBVirtualServerDef(
                virtual_server_id=vs_id, name=vs_name)
            self.assert_called_with_def(update_call, expected_def)


class TestPolicyLBPoolApi(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBPoolApi, self).setUp()
        self.resourceApi = self.policy_lib.load_balancer.lb_pool

    def test_create_with_id(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        members = [
            lb_defs.LBPoolMemberDef(ip_address='10.0.0.1')]
        algorithm = 'algo'
        active_monitor_paths = 'path1'
        member_group = 'group1'
        snat_translation = False
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name,
                lb_pool_id=obj_id,
                description=description,
                members=members,
                active_monitor_paths=active_monitor_paths,
                algorithm=algorithm,
                member_group=member_group,
                snat_translation=snat_translation,
                tcp_multiplexing_enabled=True,
                tcp_multiplexing_number=10,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                name=name,
                description=description,
                members=members,
                active_monitor_paths=active_monitor_paths,
                algorithm=algorithm,
                member_group=member_group,
                snat_translation=snat_translation,
                tcp_multiplexing_enabled=True,
                tcp_multiplexing_number=10,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        description = 'desc'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name, description=description,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=mock.ANY,
                name=name,
                description=description,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_create_with_retry_stale_revision(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        members = [
            lb_defs.LBPoolMemberDef(ip_address='10.0.0.1')]
        algorithm = 'algo'
        active_monitor_paths = 'path1'
        member_group = 'group1'
        snat_translation = False
        with mock.patch.object(self.policy_api, "create_or_update",
                               side_effect=nsxlib_exc.StaleRevision
                               ) as api_call:
            with self.assertRaises(nsxlib_exc.StaleRevision):
                self.resourceApi.create_or_overwrite(
                    name,
                    lb_pool_id=obj_id,
                    description=description,
                    members=members,
                    active_monitor_paths=active_monitor_paths,
                    algorithm=algorithm,
                    member_group=member_group,
                    snat_translation=snat_translation,
                    tenant=TEST_TENANT)
                self.assertEqual(nsxlib_testcase.NSX_MAX_ATTEMPTS,
                                 api_call.call_count)

    def test_create_with_retry_pending_delete(self):
        name = 'd1'
        description = 'desc'
        obj_id = '111'
        members = [
            lb_defs.LBPoolMemberDef(ip_address='10.0.0.1')]
        algorithm = 'algo'
        active_monitor_paths = 'path1'
        member_group = 'group1'
        snat_translation = False
        with mock.patch.object(self.policy_api, "create_or_update",
                               side_effect=nsxlib_exc.NsxPendingDelete
                               ) as api_call:
            with self.assertRaises(nsxlib_exc.NsxPendingDelete):
                self.resourceApi.create_or_overwrite(
                    name,
                    lb_pool_id=obj_id,
                    description=description,
                    members=members,
                    active_monitor_paths=active_monitor_paths,
                    algorithm=algorithm,
                    member_group=member_group,
                    snat_translation=snat_translation,
                    tenant=TEST_TENANT)
                self.assertEqual(nsxlib_testcase.NSX_MAX_ATTEMPTS,
                                 api_call.call_count)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = lb_defs.LBPoolDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        members = [{'ip_address': '10.0.0.1'}]
        algorithm = 'algo'
        active_monitor_paths = ['path1']
        member_group = 'group1'
        snat_translation = False
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}), \
            mock.patch.object(self.policy_api,
                              "create_or_update") as update_call:
            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    members=members,
                                    active_monitor_paths=active_monitor_paths,
                                    algorithm=algorithm,
                                    member_group=member_group,
                                    snat_translation=snat_translation,
                                    tcp_multiplexing_enabled=True,
                                    tcp_multiplexing_number=10,
                                    tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                name=name,
                description=description,
                members=members,
                active_monitor_paths=active_monitor_paths,
                algorithm=algorithm,
                member_group=member_group,
                snat_translation=snat_translation,
                tcp_multiplexing_enabled=True,
                tcp_multiplexing_number=10,
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)

    def test_update_without_partial_patch(self):
        obj_id = '111'
        name = 'new name'
        description = 'new desc'
        members = [{'ip_address': '10.0.0.1'}]
        algorithm = 'algo'
        active_monitor_paths = ['path1']
        member_group = 'group1'
        snat_translation = False
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}), \
            mock.patch.object(self.policy_api,
                              "create_or_update") as update_call:
            self.resourceApi.update(obj_id,
                                    name=name,
                                    description=description,
                                    members=members,
                                    active_monitor_paths=active_monitor_paths,
                                    algorithm=algorithm,
                                    member_group=member_group,
                                    snat_translation=snat_translation,
                                    tenant=TEST_TENANT,
                                    allow_partial_updates=False)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                name=name,
                description=description,
                members=members,
                active_monitor_paths=active_monitor_paths,
                algorithm=algorithm,
                member_group=member_group,
                snat_translation=snat_translation,
                tenant=TEST_TENANT)
            update_call.assert_called_with(mock.ANY, partial_updates=False,
                                           force=False)
            self.assert_called_with_def(update_call, expected_def)

    def test_add_monitor_to_pool(self):
        obj_id = '111'
        active_monitor_paths = ['path1']
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}), \
            mock.patch.object(self.policy_api,
                              "create_or_update") as update_call:
            self.resourceApi.add_monitor_to_pool(
                obj_id,
                active_monitor_paths,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                active_monitor_paths=active_monitor_paths,
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)

    def test_remove_monitor_from_pool(self):
        obj_id = '111'
        removed_monitor_path = 'path1'
        stay_monitor_path = 'path2'
        active_monitors = [removed_monitor_path, stay_monitor_path]
        with mock.patch.object(
            self.policy_api, "get", return_value={
                'id': obj_id, 'active_monitor_paths': active_monitors}), \
            mock.patch.object(self.policy_api,
                              "create_or_update") as update_call:
            self.resourceApi.remove_monitor_from_pool(
                obj_id,
                removed_monitor_path,
                tenant=TEST_TENANT)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                active_monitor_paths=[stay_monitor_path],
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)

    def test_create_pool_member_and_add_to_pool(self):
        obj_id = '111'
        ip_address = '1.1.1.1'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}), \
            mock.patch.object(self.policy_api,
                              "create_or_update") as update_call:
            self.resourceApi.create_pool_member_and_add_to_pool(
                obj_id, ip_address,
                tenant=TEST_TENANT)
            mem_def = lb_defs.LBPoolMemberDef(ip_address)
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                members=[mem_def],
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)

    def test_update_pool_member(self):
        obj_id = '111'
        ip_address = '1.1.1.1'
        port = '80'
        new_name = 'mem1'
        member = {'ip_address': ip_address, 'port': port,
                  'backup_member': True}
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id,
                                             'members': [member]}), \
            mock.patch.object(self.policy_api,
                              "create_or_update") as update_call:
            self.resourceApi.update_pool_member(
                obj_id, ip_address, port=port, display_name=new_name,
                backup_member=False, tenant=TEST_TENANT)
            new_member = copy.copy(member)
            new_member['display_name'] = new_name
            new_member['backup_member'] = False
            expected_def = lb_defs.LBPoolDef(
                lb_pool_id=obj_id,
                members=[new_member],
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)

    def test_wait_until_realized_fail(self):
        pool_id = 'test_pool'
        info = {'state': constants.STATE_UNREALIZED,
                'realization_specific_identifier': pool_id}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            self.assertRaises(nsxlib_exc.RealizationTimeoutError,
                              self.resourceApi.wait_until_realized,
                              pool_id, max_attempts=5, sleep=0.1,
                              tenant=TEST_TENANT)

    def test_wait_until_realized_succeed(self):
        pool_id = 'test_pool'
        info = {'state': constants.STATE_REALIZED,
                'realization_specific_identifier': pool_id,
                'entity_type': 'LbPoolDto'}
        with mock.patch.object(self.resourceApi, "_get_realization_info",
                               return_value=info):
            actual_info = self.resourceApi.wait_until_realized(
                pool_id, entity_type='LbPoolDto', max_attempts=5,
                sleep=0.1, tenant=TEST_TENANT)
            self.assertEqual(info, actual_info)


class TestPolicyLBMonitorProfileHttpApi(test_resources.NsxPolicyLibTestCase):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBMonitorProfileHttpApi, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_monitor_profile_http)
        self.obj_def = lb_defs.LBHttpMonitorProfileDef

    def test_create_with_id(self):
        name = 'd1'
        obj_id = '111'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                lb_monitor_profile_id=obj_id,
                name=name,
                tenant=TEST_TENANT)
            expected_def = self.obj_def(
                lb_monitor_profile_id=obj_id,
                name=name,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result)

    def test_create_without_id(self):
        name = 'd1'
        with mock.patch.object(self.policy_api,
                               "create_or_update") as api_call:
            result = self.resourceApi.create_or_overwrite(
                name=name,
                tenant=TEST_TENANT)
            expected_def = self.obj_def(
                lb_monitor_profile_id=mock.ANY,
                name=name,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertIsNotNone(result)

    def test_delete(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "delete") as api_call:
            self.resourceApi.delete(obj_id, tenant=TEST_TENANT)
            expected_def = self.obj_def(
                lb_monitor_profile_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_get(self):
        obj_id = '111'
        with mock.patch.object(self.policy_api, "get",
                               return_value={'id': obj_id}) as api_call:
            result = self.resourceApi.get(obj_id, tenant=TEST_TENANT)
            expected_def = self.obj_def(
                lb_monitor_profile_id=obj_id,
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual(obj_id, result['id'])

    def test_get_by_name(self):
        name = 'd1'
        with mock.patch.object(
            self.policy_api, "list",
            return_value={'results': [{'display_name': name}]}) as api_call:
            obj = self.resourceApi.get_by_name(name, tenant=TEST_TENANT)
            self.assertIsNotNone(obj)
            expected_def = self.obj_def(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)

    def test_list(self):
        with mock.patch.object(self.policy_api, "list",
                               return_value={'results': []}) as api_call:
            result = self.resourceApi.list(tenant=TEST_TENANT)
            expected_def = self.obj_def(
                tenant=TEST_TENANT)
            self.assert_called_with_def(api_call, expected_def)
            self.assertEqual([], result)

    def test_update(self):
        obj_id = '111'
        name = 'new name'
        with self.mock_get(obj_id, name), \
            self.mock_create_update() as update_call:

            self.resourceApi.update(obj_id,
                                    name=name,
                                    tenant=TEST_TENANT)
            expected_def = self.obj_def(
                lb_monitor_profile_id=obj_id,
                name=name,
                tenant=TEST_TENANT)
            self.assert_called_with_def(update_call, expected_def)


class TestPolicyLBMonitorProfileHttpsApi(TestPolicyLBMonitorProfileHttpApi):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBMonitorProfileHttpsApi, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_monitor_profile_https)
        self.obj_def = lb_defs.LBHttpsMonitorProfileDef


class TestPolicyLBMonitorProfileUdpApi(TestPolicyLBMonitorProfileHttpApi):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBMonitorProfileUdpApi, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_monitor_profile_udp)
        self.obj_def = lb_defs.LBUdpMonitorProfileDef


class TestPolicyLBMonitorProfileIcmpApi(TestPolicyLBMonitorProfileHttpApi):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBMonitorProfileIcmpApi, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_monitor_profile_icmp)
        self.obj_def = lb_defs.LBIcmpMonitorProfileDef


class TestPolicyLBMonitorProfileTcpApi(TestPolicyLBMonitorProfileHttpApi):

    def setUp(self, *args, **kwargs):
        super(TestPolicyLBMonitorProfileTcpApi, self).setUp()
        self.resourceApi = (
            self.policy_lib.load_balancer.lb_monitor_profile_tcp)
        self.obj_def = lb_defs.LBTcpMonitorProfileDef



# Copyright 2019 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from vmware_nsxlib.v3.policy import constants
from vmware_nsxlib.v3.policy import core_defs

TENANTS_PATH_PATTERN = "%s/"
IPSEC_VPN_IKE_PROFILES_PATH_PATTERN = (TENANTS_PATH_PATTERN +
                                       "ipsec-vpn-ike-profiles/")
IPSEC_VPN_TUNNEL_PROFILES_PATH_PATTERN = (TENANTS_PATH_PATTERN +
                                          "ipsec-vpn-tunnel-profiles/")
IPSEC_VPN_DPD_PROFILES_PATH_PATTERN = (TENANTS_PATH_PATTERN +
                                       "ipsec-vpn-dpd-profiles/")
IPSEC_VPN_SERVICE_PATH_PATTERN = (
    core_defs.TIER1_LOCALE_SERVICES_PATH_PATTERN + "%s/ipsec-vpn-services/")

IPSEC_VPN_DPD_PROFILES_PATH_PATTERN = (TENANTS_PATH_PATTERN +
                                       "ipsec-vpn-dpd-profiles/")


class IpsecVpnIkeProfileDef(core_defs.ResourceDef):

    @property
    def path_pattern(self):
        return IPSEC_VPN_IKE_PROFILES_PATH_PATTERN

    @property
    def path_ids(self):
        return ('tenant', 'profile_id')

    @staticmethod
    def resource_type():
        return "IPSecVpnIkeProfile"

    def get_obj_dict(self):
        body = super(IpsecVpnIkeProfileDef, self).get_obj_dict()
        self._set_attrs_if_specified(body, ["ike_version",
                                            "encryption_algorithms",
                                            "digest_algorithms",
                                            "dh_groups",
                                            "sa_life_time"])
        return body


class IpsecVpnTunnelProfileDef(core_defs.ResourceDef):

    @property
    def path_pattern(self):
        return IPSEC_VPN_TUNNEL_PROFILES_PATH_PATTERN

    @property
    def path_ids(self):
        return ('tenant', 'profile_id')

    @staticmethod
    def resource_type():
        return "IPSecVpnTunnelProfile"

    def get_obj_dict(self):
        body = super(IpsecVpnTunnelProfileDef, self).get_obj_dict()
        self._set_attrs_if_specified(body, ["enable_perfect_forward_secrecy",
                                            "encryption_algorithms",
                                            "digest_algorithms",
                                            "dh_groups",
                                            "sa_life_time"])
        return body


class IpsecVpnDpdProfileDef(core_defs.ResourceDef):

    @property
    def path_pattern(self):
        return IPSEC_VPN_DPD_PROFILES_PATH_PATTERN

    @property
    def path_ids(self):
        return ('tenant', 'profile_id')

    @staticmethod
    def resource_type():
        return "IPSecVpnDpdProfile"

    def get_obj_dict(self):
        body = super(IpsecVpnDpdProfileDef, self).get_obj_dict()
        self._set_attrs_if_specified(body, ["dpd_probe_interval", "enabled"])
        return body


class Tier1IPSecVpnServiceDef(core_defs.ResourceDef):

    @staticmethod
    def resource_type():
        return 'IPSecVpnService'

    @property
    def path_pattern(self):
        return IPSEC_VPN_SERVICE_PATH_PATTERN

    def get_obj_dict(self):
        body = super(Tier1IPSecVpnServiceDef, self).get_obj_dict()
        self._set_attrs_if_specified(body, ['enabled', 'ike_log_level'])
        return body

    @property
    def path_ids(self):
        return ('tenant', 'tier1_id', 'service_id', 'vpn_service_id')


class IpsecVpnLocalEndpointDef(core_defs.ResourceDef):

    @property
    def path_pattern(self):
        return IPSEC_VPN_SERVICE_PATH_PATTERN + "%s/local-endpoints/"

    @property
    def path_ids(self):
        return ('tenant', 'tier1_id', 'service_id', 'vpn_service_id',
                'endpoint_id')

    @staticmethod
    def resource_type():
        return "IPSecVpnLocalEndpoint"

    def get_obj_dict(self):
        body = super(IpsecVpnLocalEndpointDef, self).get_obj_dict()
        self._set_attrs_if_specified(body, ["local_address", "local_id",
                                            "certificate_path",
                                            "trust_ca_ids", "trust_crl_ids"])
        return body


class IPSecVpnRule(object):

    def __init__(self, name, rule_id, action=constants.IPSEC_VPN_RULE_PROTECT,
                 description=None, enabled=True, logged=False,
                 destination_cidrs=None, source_cidrs=None, sequence_number=0,
                 tags=None):
        self.name = name
        self.description = description
        self.action = action
        self.enabled = enabled
        self.id = rule_id
        self.logged = logged
        self.destination_cidrs = destination_cidrs
        self.source_cidrs = source_cidrs
        self.sequence_number = sequence_number
        self.tags = tags

    def get_obj_dict(self):
        obj = {'display_name': self.name,
               'id': self.id,
               'action': self.action,
               'enabled': self.enabled,
               'logged': self.logged,
               'resource_type': 'IPSecVpnRule'}

        if self.description:
            obj['description'] = self.description

        if self.destination_cidrs:
            obj['destinations'] = [
                {'subnet': cidr} for cidr in self.destination_cidrs]

        if self.source_cidrs:
            obj['sources'] = [
                {'subnet': cidr} for cidr in self.source_cidrs]

        if self.sequence_number:
            obj['sequence_number'] = self.sequence_number

        if self.tags:
            obj['tags'] = self.tags

        return obj


class Tier1IPSecVpnSessionDef(core_defs.ResourceDef):

    @staticmethod
    def resource_type():
        return 'PolicyBasedIPSecVpnSession'

    @property
    def path_pattern(self):
        return IPSEC_VPN_SERVICE_PATH_PATTERN + "%s/sessions/"

    def get_obj_dict(self):
        body = super(Tier1IPSecVpnSessionDef, self).get_obj_dict()
        self._set_attrs_if_specified(body, ['enabled', 'peer_address',
                                            'peer_id', 'psk'])

        if self.has_attr('rules'):
            body['rules'] = [
                a.get_obj_dict()
                if isinstance(a, IPSecVpnRule) else a
                for a in self.get_attr('rules')]

        if self.has_attr('dpd_profile_id'):
            path = ""
            if self.get_attr('dpd_profile_id'):
                profile = IpsecVpnDpdProfileDef(
                    profile_id=self.get_attr('dpd_profile_id'),
                    tenant=self.get_tenant())
                path = profile.get_resource_full_path()
            self._set_attr_if_specified(body, 'dpd_profile_id',
                                        body_attr='dpd_profile_path',
                                        value=path)

        if self.has_attr('ike_profile_id'):
            path = ""
            if self.get_attr('ike_profile_id'):
                profile = IpsecVpnIkeProfileDef(
                    profile_id=self.get_attr('ike_profile_id'),
                    tenant=self.get_tenant())
                path = profile.get_resource_full_path()
            self._set_attr_if_specified(body, 'ike_profile_id',
                                        body_attr='ike_profile_path',
                                        value=path)

        if self.has_attr('tunnel_profile_id'):
            path = ""
            if self.get_attr('tunnel_profile_id'):
                profile = IpsecVpnTunnelProfileDef(
                    profile_id=self.get_attr('tunnel_profile_id'),
                    tenant=self.get_tenant())
                path = profile.get_resource_full_path()
            self._set_attr_if_specified(body, 'tunnel_profile_id',
                                        body_attr='tunnel_profile_path',
                                        value=path)

        if self.has_attr('local_endpoint_id'):
            path = ""
            if self.get_attr('local_endpoint_id'):
                endpoint = IpsecVpnLocalEndpointDef(
                    tier1_id=self.get_attr('tier1_id'),
                    service_id=self.get_attr('service_id'),
                    vpn_service_id=self.get_attr('vpn_service_id'),
                    endpoint_id=self.get_attr('local_endpoint_id'),
                    tenant=self.get_tenant())
                path = endpoint.get_resource_full_path()
            self._set_attr_if_specified(body, 'local_endpoint_id',
                                        body_attr='local_endpoint_path',
                                        value=path)
        return body

    @property
    def path_ids(self):
        return ('tenant', 'tier1_id', 'service_id', 'vpn_service_id',
                'session_id')


class Tier1IPSecVpnSessionStatusDef(core_defs.ResourceDef):

    @property
    def path_pattern(self):
        return (IPSEC_VPN_SERVICE_PATH_PATTERN +
                "%s/sessions/%s/detailed-status/")

    @property
    def path_ids(self):
        # TODO(asarfaty): Why do we need the last '' entry? Need to find a
        # better solution for this.
        return ('tenant', 'tier1_id', 'service_id', 'vpn_service_id',
                'session_id', '')



# Copyright 2019 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from oslo_log import log as logging

from vmware_nsxlib.v3.policy import constants
from vmware_nsxlib.v3.policy import core_resources
from vmware_nsxlib.v3.policy import ipsec_vpn_defs

LOG = logging.getLogger(__name__)
IGNORE = core_resources.IGNORE


class NsxIpsecVpnIkeProfileApi(core_resources.NsxPolicyResourceBase):
    @property
    def entry_def(self):
        return ipsec_vpn_defs.IpsecVpnIkeProfileDef

    def create_or_overwrite(self, name,
                            profile_id=None,
                            description=IGNORE,
                            ike_version=IGNORE,
                            encryption_algorithms=IGNORE,
                            digest_algorithms=IGNORE,
                            dh_groups=IGNORE,
                            sa_life_time=IGNORE,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):

        profile_id = self._init_obj_uuid(profile_id)
        profile_def = self._init_def(
            profile_id=profile_id,
            name=name,
            description=description,
            ike_version=ike_version,
            encryption_algorithms=encryption_algorithms,
            digest_algorithms=digest_algorithms,
            dh_groups=dh_groups,
            sa_life_time=sa_life_time,
            tags=tags,
            tenant=tenant)
        self._create_or_store(profile_def)
        return profile_id

    def delete(self, profile_id, tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(profile_id=profile_id,
                                     tenant=tenant)
        self._delete_with_retry(profile_def)

    def get(self, profile_id, tenant=constants.POLICY_INFRA_TENANT,
            silent=False):
        profile_def = self.entry_def(profile_id=profile_id,
                                     tenant=tenant)
        return self.policy_api.get(profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(tenant=tenant)
        return self._list(profile_def)

    def get_by_name(self, name, tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxIpsecVpnIkeProfileApi, self).get_by_name(
            name, tenant=tenant)

    def update(self, profile_id, name=IGNORE, description=IGNORE,
               ike_version=IGNORE, encryption_algorithms=IGNORE,
               digest_algorithms=IGNORE, dh_groups=IGNORE, sa_life_time=IGNORE,
               tags=IGNORE, tenant=constants.POLICY_INFRA_TENANT):
        self._update(profile_id=profile_id,
                     name=name,
                     description=description,
                     ike_version=ike_version,
                     encryption_algorithms=encryption_algorithms,
                     digest_algorithms=digest_algorithms,
                     dh_groups=dh_groups,
                     sa_life_time=sa_life_time,
                     tags=tags,
                     tenant=tenant)


class NsxIpsecVpnTunnelProfileApi(core_resources.NsxPolicyResourceBase):
    @property
    def entry_def(self):
        return ipsec_vpn_defs.IpsecVpnTunnelProfileDef

    def create_or_overwrite(self, name,
                            profile_id=None,
                            description=IGNORE,
                            enable_perfect_forward_secrecy=IGNORE,
                            encryption_algorithms=IGNORE,
                            digest_algorithms=IGNORE,
                            dh_groups=IGNORE,
                            sa_life_time=IGNORE,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):

        profile_id = self._init_obj_uuid(profile_id)
        profile_def = self._init_def(
            profile_id=profile_id,
            name=name,
            description=description,
            enable_perfect_forward_secrecy=enable_perfect_forward_secrecy,
            encryption_algorithms=encryption_algorithms,
            digest_algorithms=digest_algorithms,
            dh_groups=dh_groups,
            sa_life_time=sa_life_time,
            tags=tags,
            tenant=tenant)
        self._create_or_store(profile_def)
        return profile_id

    def delete(self, profile_id, tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(profile_id=profile_id,
                                     tenant=tenant)
        self._delete_with_retry(profile_def)

    def get(self, profile_id, tenant=constants.POLICY_INFRA_TENANT,
            silent=False):
        profile_def = self.entry_def(profile_id=profile_id,
                                     tenant=tenant)
        return self.policy_api.get(profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(tenant=tenant)
        return self._list(profile_def)

    def get_by_name(self, name, tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxIpsecVpnTunnelProfileApi, self).get_by_name(
            name, tenant=tenant)

    def update(self, profile_id, name=IGNORE, description=IGNORE,
               enable_perfect_forward_secrecy=IGNORE,
               encryption_algorithms=IGNORE,
               digest_algorithms=IGNORE, dh_groups=IGNORE, sa_life_time=IGNORE,
               tags=IGNORE, tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            profile_id=profile_id,
            name=name,
            description=description,
            enable_perfect_forward_secrecy=enable_perfect_forward_secrecy,
            encryption_algorithms=encryption_algorithms,
            digest_algorithms=digest_algorithms,
            dh_groups=dh_groups,
            sa_life_time=sa_life_time,
            tags=tags,
            tenant=tenant)


class NsxIpsecVpnDpdProfileApi(core_resources.NsxPolicyResourceBase):
    @property
    def entry_def(self):
        return ipsec_vpn_defs.IpsecVpnDpdProfileDef

    def create_or_overwrite(self, name,
                            profile_id=None,
                            description=IGNORE,
                            dpd_probe_interval=IGNORE,
                            enabled=IGNORE,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):

        profile_id = self._init_obj_uuid(profile_id)
        profile_def = self._init_def(
            profile_id=profile_id,
            name=name,
            description=description,
            dpd_probe_interval=dpd_probe_interval,
            enabled=enabled,
            tags=tags,
            tenant=tenant)
        self._create_or_store(profile_def)
        return profile_id

    def delete(self, profile_id, tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(profile_id=profile_id,
                                     tenant=tenant)
        self._delete_with_retry(profile_def)

    def get(self, profile_id, tenant=constants.POLICY_INFRA_TENANT,
            silent=False):
        profile_def = self.entry_def(profile_id=profile_id,
                                     tenant=tenant)
        return self.policy_api.get(profile_def, silent=silent)

    def list(self, tenant=constants.POLICY_INFRA_TENANT):
        profile_def = self.entry_def(tenant=tenant)
        return self._list(profile_def)

    def get_by_name(self, name, tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxIpsecVpnDpdProfileApi, self).get_by_name(
            name, tenant=tenant)

    def update(self, profile_id, name=IGNORE, description=IGNORE,
               dpd_probe_interval=IGNORE, enabled=IGNORE,
               tags=IGNORE, tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            profile_id=profile_id,
            name=name,
            description=description,
            dpd_probe_interval=dpd_probe_interval,
            enabled=enabled,
            tags=tags,
            tenant=tenant)


class NsxIpsecVpnServiceApi(core_resources.NsxPolicyResourceBase):
    @property
    def entry_def(self):
        return ipsec_vpn_defs.Tier1IPSecVpnServiceDef

    def _locale_service_id(self, tier1_id):
        return core_resources.NsxPolicyTier1Api._locale_service_id(tier1_id)

    def create_or_overwrite(self, name, tier1_id,
                            vpn_service_id=None,
                            description=IGNORE,
                            enabled=IGNORE,
                            ike_log_level=IGNORE,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):

        vpn_service_id = self._init_obj_uuid(vpn_service_id)
        service_def = self._init_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            name=name,
            description=description,
            enabled=enabled,
            ike_log_level=ike_log_level,
            tags=tags,
            tenant=tenant)
        self._create_or_store(service_def)
        return vpn_service_id

    def delete(self, tier1_id, vpn_service_id,
               tenant=constants.POLICY_INFRA_TENANT):
        service_def = self.entry_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            tenant=tenant)
        self._delete_with_retry(service_def)

    def get(self, tier1_id, vpn_service_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        service_def = self.entry_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            tenant=tenant)
        return self.policy_api.get(service_def, silent=silent)

    def list(self, tier1_id, tenant=constants.POLICY_INFRA_TENANT):
        service_def = self.entry_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            tenant=tenant)
        return self._list(service_def)

    def get_by_name(self, tier1_id, name,
                    tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxIpsecVpnServiceApi, self).get_by_name(
            name, tier1_id=tier1_id,
            tenant=tenant)

    def update(self, tier1_id, vpn_service_id, name=IGNORE, description=IGNORE,
               enabled=IGNORE, ike_log_level=IGNORE,
               tags=IGNORE, tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            name=name,
            description=description,
            enabled=enabled,
            ike_log_level=ike_log_level,
            tags=tags,
            tenant=tenant)


class NsxIpsecVpnLocalEndpointApi(core_resources.NsxPolicyResourceBase):
    @property
    def entry_def(self):
        return ipsec_vpn_defs.IpsecVpnLocalEndpointDef

    def _locale_service_id(self, tier1_id):
        return core_resources.NsxPolicyTier1Api._locale_service_id(tier1_id)

    def create_or_overwrite(self, name, tier1_id,
                            vpn_service_id,
                            endpoint_id=None,
                            description=IGNORE,
                            local_address=IGNORE,
                            local_id=IGNORE,
                            certificate_path=IGNORE,
                            trust_ca_ids=IGNORE,
                            trust_crl_ids=IGNORE,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):

        endpoint_id = self._init_obj_uuid(endpoint_id)
        endpoint_def = self._init_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            endpoint_id=endpoint_id,
            name=name,
            description=description,
            local_address=local_address,
            local_id=local_id,
            certificate_path=certificate_path,
            trust_ca_ids=trust_ca_ids,
            trust_crl_ids=trust_crl_ids,
            tags=tags,
            tenant=tenant)
        self._create_or_store(endpoint_def)
        return endpoint_id

    def delete(self, tier1_id, vpn_service_id, endpoint_id,
               tenant=constants.POLICY_INFRA_TENANT):
        endpoint_def = self.entry_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            endpoint_id=endpoint_id,
            tenant=tenant)
        self._delete_with_retry(endpoint_def)

    def get(self, tier1_id, vpn_service_id, endpoint_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        endpoint_def = self.entry_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            endpoint_id=endpoint_id,
            tenant=tenant)
        return self.policy_api.get(endpoint_def, silent=silent)

    def list(self, tier1_id, vpn_service_id,
             tenant=constants.POLICY_INFRA_TENANT):
        endpoint_def = self.entry_def(
            tier1_id=tier1_id,
            vpn_service_id=vpn_service_id,
            service_id=self._locale_service_id(tier1_id),
            tenant=tenant)
        return self._list(endpoint_def)

    def get_by_name(self, tier1_id, vpn_service_id, name,
                    tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxIpsecVpnLocalEndpointApi, self).get_by_name(
            name, tier1_id=tier1_id,
            vpn_service_id=vpn_service_id,
            tenant=tenant)

    def update(self, tier1_id, vpn_service_id, endpoint_id,
               name=IGNORE,
               description=IGNORE,
               local_address=IGNORE,
               local_id=IGNORE,
               certificate_path=IGNORE,
               trust_ca_ids=IGNORE,
               trust_crl_ids=IGNORE,
               tags=IGNORE, tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            endpoint_id=endpoint_id,
            name=name,
            description=description,
            local_address=local_address,
            local_id=local_id,
            certificate_path=certificate_path,
            trust_ca_ids=trust_ca_ids,
            trust_crl_ids=trust_crl_ids,
            tags=tags,
            tenant=tenant)


class NsxIpsecVpnSessionApi(core_resources.NsxPolicyResourceBase):
    @property
    def entry_def(self):
        return ipsec_vpn_defs.Tier1IPSecVpnSessionDef

    def _locale_service_id(self, tier1_id):
        return core_resources.NsxPolicyTier1Api._locale_service_id(tier1_id)

    def create_or_overwrite(self, name, tier1_id,
                            vpn_service_id,
                            session_id=None,
                            description=IGNORE,
                            enabled=IGNORE,
                            peer_address=IGNORE,
                            peer_id=IGNORE,
                            psk=IGNORE,
                            rules=IGNORE,
                            dpd_profile_id=IGNORE,
                            ike_profile_id=IGNORE,
                            tunnel_profile_id=IGNORE,
                            local_endpoint_id=IGNORE,
                            tags=IGNORE,
                            tenant=constants.POLICY_INFRA_TENANT):

        session_id = self._init_obj_uuid(session_id)
        session_def = self._init_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            session_id=session_id,
            name=name,
            description=description,
            enabled=enabled,
            peer_address=peer_address,
            peer_id=peer_id,
            psk=psk,
            rules=rules,
            dpd_profile_id=dpd_profile_id,
            ike_profile_id=ike_profile_id,
            tunnel_profile_id=tunnel_profile_id,
            local_endpoint_id=local_endpoint_id,
            tags=tags,
            tenant=tenant)
        self._create_or_store(session_def)
        return session_id

    def delete(self, tier1_id, vpn_service_id, session_id,
               tenant=constants.POLICY_INFRA_TENANT):
        session_def = self.entry_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            session_id=session_id,
            tenant=tenant)
        self._delete_with_retry(session_def)

    def get(self, tier1_id, vpn_service_id, session_id,
            tenant=constants.POLICY_INFRA_TENANT, silent=False):
        session_def = self.entry_def(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            session_id=session_id,
            tenant=tenant)
        return self.policy_api.get(session_def, silent=silent)

    def get_status(self, tier1_id, vpn_service_id, session_id,
                   tenant=constants.POLICY_INFRA_TENANT):
        status_def = ipsec_vpn_defs.Tier1IPSecVpnSessionStatusDef(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            session_id=session_id,
            tenant=tenant)
        return self.policy_api.get(status_def)

    def list(self, tier1_id, vpn_service_id,
             tenant=constants.POLICY_INFRA_TENANT):
        session_def = self.entry_def(
            tier1_id=tier1_id,
            vpn_service_id=vpn_service_id,
            service_id=self._locale_service_id(tier1_id),
            tenant=tenant)
        return self._list(session_def)

    def get_by_name(self, tier1_id, vpn_service_id, name,
                    tenant=constants.POLICY_INFRA_TENANT):
        return super(NsxIpsecVpnSessionApi, self).get_by_name(
            name, tier1_id=tier1_id,
            vpn_service_id=vpn_service_id,
            tenant=tenant)

    def update(self, tier1_id, vpn_service_id, session_id,
               name=IGNORE,
               description=IGNORE,
               enabled=IGNORE,
               peer_address=IGNORE,
               peer_id=IGNORE,
               psk=IGNORE,
               rules=IGNORE,
               dpd_profile_id=IGNORE,
               ike_profile_id=IGNORE,
               tunnel_profile_id=IGNORE,
               local_endpoint_id=IGNORE,
               tags=IGNORE, tenant=constants.POLICY_INFRA_TENANT):
        self._update(
            tier1_id=tier1_id,
            service_id=self._locale_service_id(tier1_id),
            vpn_service_id=vpn_service_id,
            session_id=session_id,
            name=name,
            description=description,
            enabled=enabled,
            peer_address=peer_address,
            peer_id=peer_id,
            psk=psk,
            rules=rules,
            dpd_profile_id=dpd_profile_id,
            ike_profile_id=ike_profile_id,
            tunnel_profile_id=tunnel_profile_id,
            local_endpoint_id=local_endpoint_id,
            tags=tags,
            tenant=tenant)

    def build_rule(self, name, rule_id,
                   action=constants.IPSEC_VPN_RULE_PROTECT,
                   description=None, enabled=True,
                   logged=False, destination_cidrs=None, source_cidrs=None,
                   sequence_number=0, tags=None):
        return ipsec_vpn_defs.IPSecVpnRule(
            name=name, action=action, description=description,
            enabled=enabled, rule_id=rule_id, logged=logged,
            destination_cidrs=destination_cidrs,
            source_cidrs=source_cidrs, sequence_number=sequence_number,
            tags=tags)


class NsxPolicyIpsecVpnApi(object):
    """This is the class that have all IPSEC VPN  policy apis"""
    def __init__(self, *args):
        self.ike_profile = NsxIpsecVpnIkeProfileApi(*args)
        self.tunnel_profile = NsxIpsecVpnTunnelProfileApi(*args)
        self.dpd_profile = NsxIpsecVpnDpdProfileApi(*args)
        self.service = NsxIpsecVpnServiceApi(*args)
        self.local_endpoint = NsxIpsecVpnLocalEndpointApi(*args)
        self.session = NsxIpsecVpnSessionApi(*args)



# Copyright 2020 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from collections import namedtuple
import logging
import traceback

import six
from urllib3.exceptions import MaxRetryError
from urllib3.exceptions import ResponseError
from urllib3.util.retry import Retry

log = logging.getLogger(__name__)

RequestHistory = namedtuple(
    "RequestHistory", ["method", "url", "error", "status", "redirect_location"]
)


class RetryDebug(Retry):
    """Class that adds debugging capabilities of Retry"""
    def __init__(self, *args, **kw):
        super(RetryDebug, self).__init__(*args, **kw)

    def increment(self, method=None, url=None, response=None, error=None,
                  _pool=None, _stacktrace=None, ):
        log.debug("Retry Increment %s", traceback.format_stack())
        if self.total is False and error:
            # Disabled, indicate to re-raise the error.
            raise six.reraise(type(error), error, _stacktrace)

        total = self.total
        if total is not None:
            total -= 1

        connect = self.connect
        read = self.read
        redirect = self.redirect
        status_count = self.status
        cause = "unknown"
        status = None
        redirect_location = None

        if error and self._is_connection_error(error):
            # Connect retry?
            if connect is False:
                raise six.reraise(type(error), error, _stacktrace)
            elif connect is not None:
                connect -= 1

        elif error and self._is_read_error(error):
            # Read retry?
            if read is False or not self._is_method_retryable(method):
                raise six.reraise(type(error), error, _stacktrace)
            elif read is not None:
                read -= 1

        elif response and response.get_redirect_location():
            # Redirect retry?
            if redirect is not None:
                redirect -= 1
            cause = "too many redirects"
            redirect_location = response.get_redirect_location()
            status = response.status

        else:
            # Incrementing because of a server error like a 500 in
            # status_forcelist and a the given method is in the whitelist
            cause = ResponseError.GENERIC_ERROR
            if response and response.status:
                if status_count is not None:
                    status_count -= 1
                cause = ResponseError.SPECIFIC_ERROR.format(
                    status_code=response.status)
                status = response.status

        history = self.history + (
            RequestHistory(method, url, error, status, redirect_location),
        )

        new_retry = self.new(
            total=total,
            connect=connect,
            read=read,
            redirect=redirect,
            status=status_count,
            history=history,
        )

        if new_retry.is_exhausted():
            raise MaxRetryError(_pool, url, error or ResponseError(cause))

        # log the cause for this retry
        log.debug("Cause for retry: %s", cause)
        # log the server response for this retry
        log.debug("Response: %s", response)
        log.debug("Incremented Retry for (url='%s'): %r", url, new_retry)

        return new_retry



# Copyright 2015 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import re
import time
from urllib import parse as urlparse

from oslo_log import log
from oslo_serialization import jsonutils
import requests

from vmware_nsxlib._i18n import _
from vmware_nsxlib.v3 import exceptions
from vmware_nsxlib.v3 import utils

LOG = log.getLogger(__name__)

NULL_CURSOR_PREFIX = '0000'


def get_http_error_details(response):
    msg = response.json() if response.content else ''
    error_code = None
    related_error_codes = []
    related_status_codes = []

    if isinstance(msg, dict) and 'error_message' in msg:
        error_code = msg.get('error_code')
        related_errors = [error['error_message'] for error in
                          msg.get('related_errors', [])]
        related_error_codes = [str(error['error_code']) for error in
                               msg.get('related_errors', []) if
                               error.get('error_code')]
        related_status_codes = [getattr(requests.codes, error['httpStatus'])
                                for error in msg.get('related_errors', []) if
                                error.get('httpStatus')]

        msg = msg['error_message']
        if related_errors:
            msg += " relatedErrors: %s" % ' '.join(related_errors)

    return {'status_code': response.status_code,
            'error_code': error_code,
            'related_error_codes': related_error_codes,
            'related_status_codes': related_status_codes,
            'details': msg}


def init_http_exception_from_response(response):
    if response is None or response:
        # The response object has a __bool__ method that return True for
        # status code under 400. In that case there is no need for exception
        return None

    error_details = get_http_error_details(response)
    if not error_details['error_code']:
        return None

    error = http_error_to_exception(error_details['status_code'],
                                    error_details['error_code'],
                                    error_details['related_error_codes'])

    return error(manager='', **error_details)


def http_error_to_exception(status_code, error_code, related_error_codes=None):
    errors = {
        requests.codes.NOT_FOUND:
            {'202': exceptions.BackendResourceNotFound,
             '500090': exceptions.StaleRevision,
             'default': exceptions.ResourceNotFound},
        requests.codes.BAD_REQUEST:
            {'60508': exceptions.NsxIndexingInProgress,
             '60514': exceptions.NsxSearchTimeout,
             '60515': exceptions.NsxSearchOutOfSync,
             '8327': exceptions.NsxOverlapVlan,
             '500045': exceptions.NsxPendingDelete,
             '500030': exceptions.ResourceInUse,
             '500087': exceptions.StaleRevision,
             '500105': exceptions.NsxOverlapAddresses,
             '500232': exceptions.StaleRevision,  # Missing dependent objects
             '503040': exceptions.NsxSegemntWithVM,
             '100148': exceptions.StaleRevision,
             '500012': exceptions.NsxInvalidPath},
        requests.codes.CONFLICT: exceptions.StaleRevision,
        requests.codes.PRECONDITION_FAILED: exceptions.StaleRevision,
        requests.codes.INTERNAL_SERVER_ERROR:
            {'98': exceptions.CannotConnectToServer,
             '99': exceptions.ClientCertificateNotTrusted,
             '607': exceptions.APITransactionAborted,
             '610': exceptions.APITransactionAborted},
        requests.codes.FORBIDDEN:
            {'98': exceptions.BadXSRFToken,
             '403': exceptions.InvalidCredentials,
             '505': exceptions.InvalidLicense},
        requests.codes.TOO_MANY_REQUESTS: exceptions.TooManyRequests,
        requests.codes.SERVICE_UNAVAILABLE: exceptions.ServiceUnavailable}

    if status_code in errors:
        if isinstance(errors[status_code], dict):
            # choose based on error code
            if error_code and str(error_code) in errors[status_code]:
                return errors[status_code][str(error_code)]
            # try the related errors
            if related_error_codes:
                for err in related_error_codes:
                    if err and str(err) in errors[status_code]:
                        return errors[status_code][str(err)]

            if 'default' in errors[status_code]:
                return errors[status_code]['default']
        else:
            return errors[status_code]

    # default exception
    return exceptions.ManagerError


class RESTClient(object):

    _VERB_RESP_CODES = {
        'get': [requests.codes.ok],
        'post': [requests.codes.created, requests.codes.ok],
        'put': [requests.codes.created, requests.codes.ok],
        'patch': [requests.codes.created, requests.codes.ok],
        'delete': [requests.codes.ok]
    }

    def __init__(self, connection, url_prefix=None,
                 default_headers=None,
                 client_obj=None):
        self._conn = connection
        self._url_prefix = url_prefix or ""
        self._default_headers = default_headers or {}

    def new_client_for(self, *uri_segments):
        uri = self._build_url('/'.join(uri_segments))

        return self.__class__(
            self._conn,
            url_prefix=uri,
            default_headers=self._default_headers,
            client_obj=self)

    def list(self, resource='', headers=None, silent=False):
        return self.url_list(resource, headers=headers, silent=silent)

    def get(self, uuid, headers=None, silent=False, with_retries=False):
        return self.url_get(uuid, headers=headers, silent=silent,
                            with_retries=with_retries)

    def delete(self, uuid, headers=None, expected_results=None):
        return self.url_delete(uuid, headers=headers,
                               expected_results=expected_results)

    def update(self, uuid, body=None, headers=None, expected_results=None):
        return self.url_put(uuid, body, headers=headers,
                            expected_results=expected_results)

    def create(self, resource='', body=None, headers=None,
               expected_results=None):
        return self.url_post(resource, body, headers=headers,
                             expected_results=expected_results)

    def patch(self, resource='', body=None, headers=None):
        return self.url_patch(resource, body, headers=headers)

    def url_list(self, url, headers=None, silent=False):
        concatenate_response = self.url_get(url, headers=headers,
                                            silent=silent)
        cursor = concatenate_response.get('cursor', NULL_CURSOR_PREFIX)
        op = '&' if urlparse.urlparse(url).query else '?'
        url += op + 'cursor='

        while cursor and not cursor.startswith(NULL_CURSOR_PREFIX):
            page = self.url_get(url + cursor, headers=headers, silent=silent)
            concatenate_response['results'].extend(page.get('results', []))
            cursor = page.get('cursor', NULL_CURSOR_PREFIX)
        return concatenate_response

    def url_get(self, url, headers=None, silent=False, with_retries=False):
        return self._rest_call(url, method='GET', headers=headers,
                               silent=silent, with_retries=with_retries)

    def url_delete(self, url, headers=None, expected_results=None):
        return self._rest_call(url, method='DELETE', headers=headers,
                               expected_results=expected_results)

    def url_put(self, url, body, headers=None, expected_results=None):
        return self._rest_call(url, method='PUT', body=body, headers=headers,
                               expected_results=expected_results)

    def url_post(self, url, body, headers=None, expected_results=None):
        return self._rest_call(url, method='POST', body=body, headers=headers,
                               expected_results=expected_results)

    def url_patch(self, url, body, headers=None):
        return self._rest_call(url, method='PATCH', body=body, headers=headers)

    def _raise_error(self, operation, status_code, details,
                     error_code=None, related_error_codes=None,
                     related_status_codes=None):
        error = http_error_to_exception(status_code, error_code,
                                        related_error_codes)
        raise error(manager='', operation=operation, details=details,
                    error_code=error_code,
                    related_error_codes=related_error_codes,
                    status_code=status_code,
                    related_status_codes=related_status_codes)

    def _validate_result(self, result, expected, operation, silent=False):
        if result.status_code not in expected:
            result_msg = result.json() if result.content else ''
            if not silent:
                LOG.warning("The HTTP request returned error code "
                            "%(result)s, whereas %(expected)s response "
                            "codes were expected. Response body %(body)s",
                            {'result': result.status_code,
                             'expected': '/'.join([str(code)
                                                   for code in expected]),
                             'body': result_msg})

            error_details = get_http_error_details(result)

            self._raise_error(operation, **error_details)

    @classmethod
    def merge_headers(cls, *headers):
        merged = {}
        for header in headers:
            if header:
                merged.update(header)
        return merged

    def _build_url(self, uri):
        prefix = urlparse.urlparse(self._url_prefix)
        uri = ("/%s/%s" % (prefix.path, uri)).replace('//', '/').strip('/')
        if prefix.netloc:
            uri = "%s/%s" % (prefix.netloc, uri)
        if prefix.scheme:
            uri = "%s://%s" % (prefix.scheme, uri)
        return uri

    def _mask_password(self, json):
        '''Mask password value in json format'''
        if not json:
            return json

        pattern = r'\"password\": [^,}]*'
        return re.sub(pattern, '"password": "********"', json)

    def _rest_call(self, url, method='GET', body=None, headers=None,
                   silent=False, expected_results=None, **kwargs):
        request_headers = headers.copy() if headers else {}
        request_headers.update(self._default_headers)

        if utils.INJECT_HEADERS_CALLBACK:
            inject_headers = utils.INJECT_HEADERS_CALLBACK()
            request_headers.update(inject_headers)

        request_url = self._build_url(url)
        do_request = getattr(self._conn, method.lower())
        if silent:
            self._conn.set_silent(True)
        ts = time.time()
        result = do_request(
            request_url,
            data=body,
            headers=request_headers)
        te = time.time()
        if silent:
            self._conn.set_silent(False)

        if not silent:
            LOG.debug("[%x] REST call: %s %s. Headers: %s. Body: %s. "
                      "Response: %s. Took %2.4f",
                      id(self._conn), method, request_url,
                      utils.censor_headers(request_headers),
                      self._mask_password(body),
                      result.json() if result.content else '',
                      te - ts)

        if not expected_results:
            expected_results = RESTClient._VERB_RESP_CODES[method.lower()]
        self._validate_result(
            result, expected_results,
            _("%(verb)s %(url)s") % {'verb': method, 'url': request_url},
            silent=silent)
        return result


class JSONRESTClient(RESTClient):

    _DEFAULT_HEADERS = {
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }

    def __init__(self, connection, url_prefix=None,
                 default_headers=None,
                 client_obj=None):

        super(JSONRESTClient, self).__init__(
            connection,
            url_prefix=url_prefix,
            default_headers=RESTClient.merge_headers(
                JSONRESTClient._DEFAULT_HEADERS, default_headers),
            client_obj=None)

    def _rest_call(self, *args, **kwargs):
        if kwargs.get('body') is not None:
            kwargs['body'] = jsonutils.dumps(kwargs['body'], sort_keys=True)
        result = super(JSONRESTClient, self)._rest_call(*args, **kwargs)
        return result.json() if result.content else result


class NSX3Client(JSONRESTClient):

    NSX_V1_API_PREFIX = 'api/v1/'
    NSX_POLICY_V1_API_PREFIX = 'policy/api/v1/'

    # NOTE: For user-facing client, NsxClusteredAPI instance
    # will be passed as connection parameter below, thus all
    # requests on this client will pass via cluster code to
    # determine endpoint
    # For validation client, TimeoutSession with specific
    # endpoint parameters will be passed as connection.
    def __init__(self, connection, url_prefix=None,
                 default_headers=None,
                 nsx_api_managers=None,
                 max_attempts=utils.DEFAULT_MAX_ATTEMPTS,
                 rate_limit_retry=True,
                 client_obj=None,
                 url_path_base=NSX_V1_API_PREFIX):

        # If the client obj is defined - copy configuration from it
        if client_obj:
            self.nsx_api_managers = client_obj.nsx_api_managers or []
            self.max_attempts = client_obj.max_attempts
            self.rate_limit_retry = client_obj.rate_limit_retry
        else:
            self.nsx_api_managers = nsx_api_managers or []
            self.max_attempts = max_attempts
            self.rate_limit_retry = rate_limit_retry

        url_prefix = url_prefix or url_path_base
        if url_prefix and url_path_base not in url_prefix:
            if url_prefix.startswith('http'):
                url_prefix += '/' + url_path_base
            else:
                url_prefix = "%s/%s" % (url_path_base,
                                        url_prefix or '')

        super(NSX3Client, self).__init__(
            connection, url_prefix=url_prefix,
            default_headers=default_headers,
            client_obj=client_obj)

    def _raise_error(self, operation, status_code, details,
                     error_code=None, related_error_codes=None,
                     related_status_codes=None):
        """Override the Rest client errors to add the manager IPs"""
        error = http_error_to_exception(status_code, error_code,
                                        related_error_codes)
        raise error(manager=self.nsx_api_managers,
                    operation=operation,
                    details=details,
                    error_code=error_code,
                    related_error_codes=related_error_codes,
                    related_status_codes=related_status_codes,
                    status_code=status_code)

    def _rest_call(self, url, **kwargs):
        if 'with_retries' in kwargs and kwargs['with_retries']:
            LOG.warning("with_retries setting is deprecated and will be "
                        "removed. Please use exceptions setting in nsxlib "
                        "config instead")

        return super(NSX3Client, self)._rest_call(url, **kwargs)



# Copyright 2016 VMware, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Admin statuses
ADMIN_STATE_UP = "UP"
ADMIN_STATE_DOWN = "DOWN"

# Replication modes
MTEP = "MTEP"

# Port attachment types
ATTACHMENT_VIF = "VIF"
ATTACHMENT_LR = "LOGICALROUTER"
ATTACHMENT_DHCP = "DHCP_SERVICE"
ATTACHMENT_MDPROXY = "METADATA_PROXY"

VIF_RESOURCE_TYPE = "VifAttachmentContext"

VIF_TYPE_PARENT = "PARENT"
VIF_TYPE_CHILD = "CHILD"

ALLOCATE_ADDRESS_NONE = "None"

# SegmentPort init_state types
INIT_STATE_UNBLOCKED_VLAN = 'UNBLOCKED_VLAN'
INIT_STATE_RESTORE_VIF = 'RESTORE_VIF'

# NSXv3 L2 Gateway constants
BRIDGE_ENDPOINT = "BRIDGEENDPOINT"
FAILOVER_MODE_PREEMPTIVE = "PREEMPTIVE"
FAILOVER_MODE_NONPREEMPTIVE = "NON_PREEMPTIVE"

# Router type
ROUTER_TYPE_TIER0 = "TIER0"
ROUTER_TYPE_TIER1 = "TIER1"
ROUTER_TYPE_TIER0_DR = "DISTRIBUTED_ROUTER_TIER0"
ROUTER_TYPE_TIER1_DR = "DISTRIBUTED_ROUTER_TIER1"

LROUTERPORT_UPLINK = "LogicalRouterUpLinkPort"
LROUTERPORT_DOWNLINK = "LogicalRouterDownLinkPort"
LROUTERPORT_CENTRALIZED = "LogicalRouterCentralizedServicePort"
LROUTERPORT_LINKONTIER0 = "LogicalRouterLinkPortOnTIER0"
LROUTERPORT_LINKONTIER1 = "LogicalRouterLinkPortOnTIER1"

# NSX service type
SERVICE_DHCP = "dhcp"

# NSX-V3 Distributed Firewall constants
IP_SET = 'IPSet'
NSGROUP = 'NSGroup'
NSGROUP_COMPLEX_EXP = 'NSGroupComplexExpression'
NSGROUP_SIMPLE_EXP = 'NSGroupSimpleExpression'
NSGROUP_TAG_EXP = 'NSGroupTagExpression'
EXCLUDE_PORT = 'Exclude-Port'

# Firewall rule position
FW_INSERT_BEFORE = 'insert_before'
FW_INSERT_AFTER = 'insert_after'
FW_INSERT_BOTTOM = 'insert_bottom'
FW_INSERT_TOP = 'insert_top'

# firewall rule actions
FW_ACTION_ALLOW = 'ALLOW'
FW_ACTION_DROP = 'DROP'
FW_ACTION_REJECT = 'REJECT'

# firewall disable/enable
FW_ENABLE = 'enable_firewall'
FW_DISABLE = 'disable_firewall'

# nsgroup members update actions
NSGROUP_ADD_MEMBERS = 'ADD_MEMBERS'
NSGROUP_REMOVE_MEMBERS = 'REMOVE_MEMBERS'

# NSServices resource types
L4_PORT_SET_NSSERVICE = 'L4PortSetNSService'
ICMP_TYPE_NSSERVICE = 'ICMPTypeNSService'
IP_PROTOCOL_NSSERVICE = 'IPProtocolNSService'

# firewall section types
FW_SECTION_LAYER3 = 'LAYER3'

TARGET_TYPE_LOGICAL_SWITCH = 'LogicalSwitch'
TARGET_TYPE_LOGICAL_PORT = 'LogicalPort'
TARGET_TYPE_IPV4ADDRESS = 'IPv4Address'
TARGET_TYPE_IPV6ADDRESS = 'IPv6Address'

# filtering operators and expressions
EQUALS = 'EQUALS'

IN = 'IN'
OUT = 'OUT'
IN_OUT = 'IN_OUT'

TCP = 'TCP'
UDP = 'UDP'
ICMPV4 = 'ICMPv4'
ICMPV6 = 'ICMPv6'
IPV4 = 'IPV4'
IPV6 = 'IPV6'
IPV4_IPV6 = 'IPV4_IPV6'

LOCAL_IP_PREFIX = 'local_ip_prefix'

# Allowed address pairs
NUM_ALLOWED_IP_ADDRESSES = 128
NUM_ALLOWED_IP_ADDRESSES_v4 = NUM_ALLOWED_IP_ADDRESSES
NUM_ALLOWED_IP_ADDRESSES_v6 = 15
MAX_STATIC_ROUTES = 26

# QoS directions egress/ingress
EGRESS = 'egress'
INGRESS = 'ingress'
EGRESS_SHAPING = 'EgressRateShaper'
INGRESS_SHAPING = 'IngressRateShaper'

# Transport zone constants
TRANSPORT_TYPE_VLAN = 'VLAN'
TRANSPORT_TYPE_OVERLAY = 'OVERLAY'
HOST_SWITCH_MODE_ENS = 'ENS'
HOST_SWITCH_MODE_STANDARD = 'STANDARD'

# Error codes returned by the backend
ERR_CODE_OBJECT_NOT_FOUND = 202
ERR_CODE_IPAM_POOL_EXHAUSTED = 5109
ERR_CODE_IPAM_SPECIFIC_IP = 5123
ERR_CODE_IPAM_IP_ALLOCATED = 5141
ERR_CODE_IPAM_IP_NOT_IN_POOL = 5110
ERR_CODE_IPAM_RANGE_MODIFY = 5602
ERR_CODE_IPAM_RANGE_DELETE = 5015
ERR_CODE_IPAM_RANGE_SHRUNK = 5016

# backend versions
NSX_VERSION_1_1_0 = '1.1.0'
NSX_VERSION_2_0_0 = '2.0.0'
NSX_VERSION_2_1_0 = '2.1.0'
NSX_VERSION_2_2_0 = '2.2.0'
NSX_VERSION_2_3_0 = '2.3.0'
NSX_VERSION_2_4_0 = '2.4.0'
NSX_VERSION_2_5_0 = '2.5.0'
NSX_VERSION_3_0_0 = '3.0.0'
NSX_VERSION_3_0_2 = '3.0.2'
NSX_VERSION_3_1_0 = '3.1.0'
NSX_VERSION_3_2_0 = '3.2.0'

# Features available depending on the NSX Manager backend version
FEATURE_MAC_LEARNING = 'MAC Learning'
FEATURE_DYNAMIC_CRITERIA = 'Dynamic criteria'
FEATURE_EXCLUDE_PORT_BY_TAG = 'Exclude Port by Tag'
FEATURE_ROUTER_FIREWALL = 'Router Firewall'
FEATURE_LOAD_BALANCER = 'Load Balancer'
FEATURE_LB_HM_RESPONSE_CODES = 'Load Balancer HM response codes'
FEATURE_DHCP_RELAY = 'DHCP Relay'
FEATURE_VLAN_ROUTER_INTERFACE = 'VLAN Router Interface'
FEATURE_RATE_LIMIT = 'Requests Rate Limit'
FEATURE_IPSEC_VPN = 'IPSec VPN'
FEATURE_ON_BEHALF_OF = 'On Behalf Of'
FEATURE_TRUNK_VLAN = 'Trunk Vlan'
FEATURE_ROUTER_TRANSPORT_ZONE = 'Router Transport Zone'
FEATURE_NO_DNAT_NO_SNAT = 'No DNAT/No SNAT'
FEATURE_ENS_WITH_SEC = 'ENS with security'
FEATURE_ENS_WITH_QOS = 'ENS with QoS'
FEATURE_ICMP_STRICT = 'Strict list of supported ICMP types and codes'
FEATURE_ROUTER_ALLOCATION_PROFILE = 'Router Allocation Profile'
FEATURE_ENABLE_STANDBY_RELOCATION = 'Router Enable standby relocation'
FEATURE_PARTIAL_UPDATES = 'Partial Update with PATCH'
FEATURE_RELAX_SCALE_VALIDATION = 'Relax Scale Validation for LbService'
FEATURE_SWITCH_HYPERBUS_MODE = 'Switch hyperbus mode with policy API'
FEATURE_GET_TZ_FROM_SWITCH = 'Get TZ endpoints from host switch'
FEATURE_ROUTE_REDISTRIBUTION_CONFIG = 'Tier0 route redistribution config'
FEATURE_CONTAINER_CLUSTER_INVENTORY = 'Container Cluster Inventory'
FEATURE_IPV6 = 'IPV6 Forwarding and Address Allocation'
FEATURE_MP2P_MIGRATION = 'MP to Policy Migration'
FEATURE_SPOOFGUARD_CIDR = 'Spoofguard IPv4 CIDR'

# Features available depending on the Policy Manager backend version
FEATURE_NSX_POLICY = 'NSX Policy'
FEATURE_NSX_POLICY_NETWORKING = 'NSX Policy Networking'
FEATURE_NSX_POLICY_MDPROXY = 'NSX Policy Metadata Proxy'
FEATURE_NSX_POLICY_DHCP = 'NSX Policy DHCP'
FEATURE_NSX_POLICY_GLOBAL_CONFIG = 'NSX Policy Global Config'
FEATURE_NSX_POLICY_ADMIN_STATE = 'NSX Policy Segment admin state'
FEATURE_NSX_POLICY_ORBAC = 'NSX Policy ORBAC'
